#!/bin/bash

# configures USB-Stick for using with cryptsetup to unlock luks automatically
# works for jessie
# Quelle: https://wiki.ubuntuusers.de/System_verschl%C3%BCsseln/Entschl%C3%BCsseln_mit_einem_USB-Schl%C3%BCssel/
# jankow@datenkollektiv.net 2016

# TODOs
#   * ConfigDatei sollte aus /usr/share/doc/dkndecryptusbkey/examples geholt werden, damit diese nicht deinstalliert
#   * wird, wenn das Paket deinstalliert werden sollte
#   * außerdem sollte das Paket noch für die Nutzung mit einem bootloader erweitert werden.


# configuration files
decryptkeyconfigdir="/etc/decryptkeydevice/"
decryptkeyconfig="/etc/decryptkeydevice/decryptkeydevice.conf"
decryptkeyscript="/etc/decryptkeydevice/decryptkeydevice_keyscript.sh"
tmpKeyfile="/root/tmpKeyfile.bin"

red="\033[0;31m"
white="\033[0m"

function partitionParameters {
	rawDev=$1
    blockSize=$(parted $rawDev unit s print| grep 'Sector size'|sed -e 's|.*: ||' -e 's|B/.*||')
	# nextPartition=$(fdisk -l $rawDev|grep ${rawDev}1|awk '{print $3}')
    nextPartition=$(parted $rawDev unit s print| grep '^ 1'|awk '{print $2}'|sed 's|s||')

	keyRange=$nextPartition
	((keyRange--))

	echo "Das Gerät belegt folgendes Device: " $rawDev
	echo "Im Folgenden die Partitionstabelle des Geräts:"
	echo ""
	# fdisk -l $rawDev
    parted $rawDev unit s print
	echo "Die Sektor-Größe der Partition beträgt " $blockSize
	echo "Nächste Partition beginnt" $nextPartition
	# fdisk -l $rawDev|grep ${rawDev}
	echo ""
	echo "Wir nutzen folgenden Bereich auf dem Gerät: 2-$keyRange"
}

function guessLuksDevice {
	luksUUID=$(cat /etc/crypttab|grep UUID|head -1 |sed 's|.*UUID=||'|cut -d" " -f1)
	luksDevice=$(readlink -f /dev/disk/by-uuid/$luksUUID)
}

function addKey {
	guessLuksDevice
	echo "Wir fügen nun den Schlüssel zu Luks hinzu"
	read -p "Bitte das Luks-Device eingeben [$luksDevice]: " luksCustomDevice
	if [ "$luksCustomDevice" != "" ]; then
		luksDevice=$luksCustomDevice
	fi
	retry=0
	while ! cryptsetup luksAddKey $luksDevice $1; do
		((retry++))
		if [ $retry -gt 5 ]; then 
			echo "Maximale Anzahl der Versuche erreicht. Breche ab"
			exit; 
		fi
	done
}

function fetchKey {
	dd if=$1 bs=512 skip=1 count=4 > $tmpKeyfile
}

function configDecrypt {
	sed -i "s|USBDISKID|$devByID|" $decryptkeyconfig
}

# Warnung
echo -e "${red}Dieses Script hilft bei der Konfiguration eines USB-Sticks"
echo "mit einem Key in einem unsichtbaren Bereich vor der ersten Partition"
echo "Achtung: Bootloader nutzen üblicherweise einen Bereich nach dem MBR"
echo "für ihr Boot-Image. Sollte auf dem Stick ein Bootloader installiert sein"
echo "wird dieser mit hoher Wahrscheinlichkeit unbrauchbar."
echo "In diesem Fall solltest du die Verschlüsselung per Hand einrichten."
echo "Du solltest wissen, was du tust!"

read -p "Wirklich weiter? Bestätigen mit \"JA\": " confirm
if [ "$confirm" != "JA" ]; then
	echo "ok, breche ab."
	echo -e "Bei Bedarf kann das Skript wieder ausgeführt werden "
	exit 0
fi

# Usb-Device auswählen
echo -e "${white}Im Folgenden eine Liste aller Platten und Sticks, die mit dem Gerät verbunden sind."
echo "Bitte wähle den Stick, der für die Entschlüsselung konfiguriert werden soll."
echo ""

diskNr=0
for i in $(ls -1 /dev/disk/by-id/); 
	do diskID[$diskNr]=$i; 
        echo "[$diskNr] $i";
        ((diskNr++));
done


echo "Bitte das Hauptgerät angeben - keine Partition."
read -p "Bitte die Nummer des Devices eingeben: " diskNr
devByID=${diskID[$diskNr]}
rawDev=$(readlink -f /dev/disk/by-id/${diskID[$diskNr]})

echo -e "${red}Wir nutzen folgendes Device: $devByID"
echo "Das entspricht: $rawDev"

echo -e "Weiter: Zufälligen Schlüssel auf USB-Stick schreiben (bestätigen mit Return)"
read

# Key auf USB-Stick schreiben
echo "Das Skript geht von einer Blockgröße von 512 Byte aus. Wenn das nicht der Fall ist"
echo "!!! Bitte abbrechen und von Hand konfigurieren!!!"
echo -e "$white"

partitionParameters $rawDev

echo ""
echo -e "${red}Folgendes Kommando wird ausgeführt, wenn du weitermachst:" 
echo "dd if=/dev/urandom of=$rawDev bs=512 seek=1 count=$keyRange"
echo -e "Wirklich schreiben? Sonst abbrechen mit STRG+C${white}"
read 

dd if=/dev/urandom of=$rawDev bs=512 seek=1 count=$keyRange

# Luks Add-Key
fetchKey $rawDev
addKey $tmpKeyfile

# for testing
# exit 0

# Konfiguration bearbeiten
read -p "Wir bearbeiten nun die Konfiguration von decrypt unter $decryptkeyconfigdir. Weiter?"
configDecrypt


# crypttab bearbeiten
if grep keyscript /etc/crypttab; then
	echo "Es sieht so aus, als ob die Crypttab bereits für ein Keyskript"
	echo "konfiguriert ist. Da kann ich nicht weiter machen. Breche ab."
	exit 0
fi

echo -e "${red}Nun muss noch die /etc/crypttab bearbeitet werden.${white}"
echo "Die crypttab sieht aktuell so aus:"
cat /etc/crypttab

echo -e "${red}Die neue Crypttab würde folgendermaßen aussehen:${white}"

sed '/#/! s|luks|luks,tries=3,keyscript=/etc/decryptkeydevice/decryptkeydevice_keyscript.sh|' /etc/crypttab

echo -e "${red}Weiter?${white}"
read

echo $(grep luks /etc/crypttab) >> /etc/crypttab # Eintrag verdoppeln
sed -i '0,/luks/ s|^|# &|' /etc/crypttab
sed -i '/#/! s|luks|luks,tries=3,keyscript=/etc/decryptkeydevice/decryptkeydevice_keyscript.sh|' /etc/crypttab


# make a backup of initramfs and update initramfs
echo -e "${red}Jetzt wird die Initramdisk neu erstellt. Weiter?${white}"
read

initramfsFile=/boot/initrd.img-$(uname -r)
cp $initramfsFile $initramfsFile.orig

echo "Wir haben eine Kopie der alten initramdisk angelegt - zur Not kann damit gebootet werden: $initramfsFile.orig"
read -p "Initramfs updaten? Weiter" 

update-initramfs -u

echo "Ok, wir sind fertig. Wenn alles gut gegangen ist, sollte das "
echo "System jetzt mit eingestecktem Stick booten."
echo "Um weitere Sticks zu konfigurieren, müssen diese dupliziert werden " 
echo "(zumindest der Schlüssel) und die ID des Sticks muss in die "
echo "$decryptkeyconfig eingetragen werden."

exit 0
